#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <stdint.h>

#include "fasthash.h"
#include "treetrie.h"
#include "critbit.h"

int main0(int argc, char *argv[]) {
  tt_arena_t a;
  int i, outer;
  tt_node_ptr_t prev = TT_EMPTY;

  setbuf(stdout, NULL);
  tt_arena_init(&a);

  for (outer = 0; outer < 10; outer++) {
    /* printf("---------------------------------------- grab/drop of %d\n", prev); */
    tt_grab(&a, prev);
    tt_drop(&a, prev);
    /* tt_arena_flush(&a); */
    /* printf("---------------------------------------- AFTER DROP of %d:\n", prev); */
    /* tt_dump_arena(&a); */
    prev = TT_EMPTY;
    /* printf("======================================== LOOP ITERATION %d\n", outer); */
    /* tt_dump_arena_summary(&a); */
    for (i = 0; i < 1000000; i++) {
      tt_node_ptr_t leaf = tt_cons_leaf(&a, TT_OK, 1001);
      tt_node_ptr_t curr = tt_cons_node(&a, 0, leaf, prev);
      /* tt_dump_arena(&a); */
      prev = curr;
    }
  }

  /* tt_dump_arena(&a); */
  tt_grab(&a, prev);
  tt_drop(&a, prev);
  /* tt_arena_flush(&a); */
  tt_dump_arena_summary(&a);

  tt_arena_done(&a);
  return EXIT_SUCCESS;
}

static void dump_mapping(void *context, tt_atom_t key, tt_node_ptr_t trie) {
  printf(" -- %u --> %u/%u\n", key, tt_ptr_idx(trie), tt_ptr_tag(trie));
}

int main(int argc, char *argv[]) {
  tt_arena_t a;
  tt_node_ptr_t curr = TT_EMPTY;
  int i;

  setbuf(stdout, NULL);
  tt_arena_init(&a);

  tt_dump_arena(&a);
  for (i = 0; i < 10; i++) {
    tt_node_ptr_t next = tt_dict_set(&a, curr, i, TT_OK);
    tt_drop(&a, curr);
    curr = tt_grab(&a, next);
    printf("\nAfter i=%d...\n", i);
    tt_dump_arena(&a);
  }

  /* tt_arena_flush(&a); */
  printf("\nFinal tree node index is %u/%u\n", tt_ptr_idx(curr), tt_ptr_tag(curr));
  tt_dump_arena(&a);

  tt_dict_foreach(&a, curr, NULL, dump_mapping);

  tt_drop(&a, curr);
  curr = TT_NO_PTR;
  /* tt_arena_flush(&a); */
  tt_dump_arena(&a);

  tt_arena_done(&a);
  return EXIT_SUCCESS;
}
